@@ -30,9 +30,15 @@
 
 // TODO:
 // - sprinkle with type-checking macros and g_assert
-// - save settings in gconf
-// - use HildonCaption in settings dialog
+// - use more HildonCaption in settings dialog?
+//   - checkbox buttons look fine without Caption
+//   - PickerButton could use a larger font
+// - check settings dialog for object/memory leaks
 // - set .so version
+// - add timers to cancel async download operations
+// - support multiple instances
+
+
 
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +56,8 @@
 
 #include <gtk/gtk.h>
 
+#include <gconf/gconf-client.h>
+
 #include <hildon/hildon.h>
 #include <libhildondesktop/libhildondesktop.h>
 
@@ -60,7 +68,9 @@
 #include "ccchh-dooris-widget.h"
 
 
-//#define DEBUG
+
+
+#define DEBUG
 
 #ifdef DEBUG
 
@@ -93,305 +103,153 @@ HD_DEFINE_PLUGIN_MODULE (CCCHHDoorIsDesktopWidget, ccchh_dooris_desktop_widget,
 
 
 
-// Code to be reworked:
-
-/*
-// TODO: is this really the right thing to do?
-// code copied from the internets
-// maybe use gtk-timers instead of "for" loop
-// running the main loop from within events
-// or constructors can lead to strange results
-
-static void gtk_sleep (unsigned int ms)
-{
-  int i;
-  fflush (stdout);
-//  ms = ms / 22;
-  if (ms == 0)
-    ms ++;
-  for (i=0; i<ms; i++)
-  {
-    usleep(1000);
-    while (gtk_events_pending ())
-      gtk_main_iteration ();
-  }
-}
-*/
-
-
-
-
-// lock fetch mutex, redraw transparent button
-static inline gboolean
-fetch_update_lock (CCCHHDoorIsDesktopWidget *self)
-{
-	if (!g_static_mutex_trylock(&(self->fetch_lock))) {
-		return FALSE;
-	}
-
-	// acknowledge button, make icon transparent
-	// TODO: not really highlighting the button
-	// also it doesn't always redraw in time
-	self->current_alpha = IMAGE_ALPHA_HIGHLIGHT;
-	force_redraw(self);
-
-	return TRUE;
-}
-
-
-// unlock fetch mutex, redraw opaque button
-static inline void
-fetch_update_unlock (CCCHHDoorIsDesktopWidget *self)
-{
-	self->current_alpha = IMAGE_ALPHA_NORMAL;
-	force_redraw(self);
-
-	g_static_mutex_unlock(&(self->fetch_lock));
-}
-
-
-// initialize async update from server, lock mutex
 static void
-fetch_update_start (CCCHHDoorIsDesktopWidget *self)
+load_settings(CCCHHDoorIsDesktopWidget *self)
 {
-	g_debug("ccchh-dooris-widget: fetch_update_start()");
+	GError *error;
 
-	if (!self->connected) {
-		g_debug("ccchh-dooris-widget: not connected, not fetching new state");
-		return;
+	error = NULL;
+
+	g_debug("ccchh-dooris-widget: load_settings()");
+
+	gchar *strval = gconf_client_get_string(self->gconf_client,
+		GCONF_DIR"/update_url", &error);
+
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value update_url: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else if (strval) {
+		if (self->url) {
+			free(self->url);
+			self->url = NULL;
+		}
+
+		self->url = gnome_vfs_make_uri_from_input(strval);
+		g_debug("ccchh-dooris-widget: load_settings: url = %s", self->url);
+
+		g_free(strval);
+		strval = NULL;
 	}
 
-	if (!fetch_update_lock(self)) {
-		g_message("ccchh-dooris-widget: fetch_lock set, not updating");
-		return;
+	gint intval = gconf_client_get_int(self->gconf_client,
+		GCONF_DIR"/update_interval", &error);
+
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value update_interval: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else if (intval) {
+		self->update = intval;
+		if (self->update < UPDATE_INTERVAL_MIN) {
+			g_warning("ccchh-dooris-widget: self->update < UPDATE_INTERVAL_MIN, maybe a bug?");
+			self->update = UPDATE_INTERVAL_MIN;
+		} else if (self->update > UPDATE_INTERVAL_MAX) {
+			g_warning("ccchh-dooris-widget: self->update > UPDATE_INTERVAL_MAX, maybe a bug?");
+			self->update = UPDATE_INTERVAL_MAX;
+		}
+		g_debug("ccchh-dooris-widget: load_settings: update = %ld", (long)self->update);
 	}
 
-	g_debug("ccchh-dooris-widget: begin update from URL: %s", self->url);
+	gboolean boolval = gconf_client_get_bool(self->gconf_client,
+		GCONF_DIR"/notify_enabled", &error);
 
-	// TODO: set timeout to cancel async open
-
-	gnome_vfs_async_open(&self->gvfs_handle, self->url, GNOME_VFS_OPEN_READ,
-		GNOME_VFS_PRIORITY_MIN, fetch_update_async_open_cb, self);
-}
-
-
-// handle async open callback, begin read
-static void
-fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
-    gpointer data)
-{
-	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
-
-	g_debug("ccchh-dooris-widget: fetch_update_async_open_cb()");
-
-	if (result != GNOME_VFS_OK) {
-		g_warning("ccchh-dooris-widget: error opening URL: %s", gnome_vfs_result_to_string (result));
-		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
-		return;
-	}
-
-/*
-	if (self->gvfs_input_buffer) {
-		g_free(self->gvfs_input_buffer);
-		self->gvfs_input_buffer = NULL;
-	}
-*/
-
-	// reuse input buffer, ensure it has at least MAX_HTTP_SIZE+1
-	if (!(self->gvfs_input_buffer = g_try_realloc(self->gvfs_input_buffer, MAX_HTTP_SIZE+1))) {
-		g_warning("ccchh-dooris-widget: failed to allocate input buffer");
-		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
-		return;
-	}
-	// initialize input buffer with zeroes, so strtok won't read beyond its end
-	memset(self->gvfs_input_buffer, 0, MAX_HTTP_SIZE+1);
-
-	// TODO: set timeout to cancel async read
-
-	gnome_vfs_async_read (handle, self->gvfs_input_buffer, MAX_HTTP_SIZE,
-		fetch_update_async_read_cb, self);
-}
-
-
-// handle async read callback, begin close, parse input
-static void
-fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
-	gpointer buffer, GnomeVFSFileSize bytes_requested, GnomeVFSFileSize bytes_read,
-	gpointer data)
-{
-	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
-
-	g_debug("ccchh-dooris-widget: fetch_update_async_read_cb()");
-
-	if (result != GNOME_VFS_OK) {
-		g_warning("ccchh-dooris-widget: read error: %s", gnome_vfs_result_to_string (result));
-		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
-		return;
-	}
-
-	g_debug("ccchh-dooris-widget: read %"GNOME_VFS_SIZE_FORMAT_STR" bytes", bytes_read);
-
-	// TODO: set timeout to cancel close
-
-	// close file
-	gnome_vfs_async_close (handle, fetch_update_async_close_cb, self);
-
-	// meanwhile, parse input buffer
-
-	gulong ts, new_change_ts, new_report_ts;
-	dooris_state_t new_state;
-	char *line, *tokctx;
-
-	new_state = DOORIS_FNORD;
-	new_change_ts = 0L;
-	new_report_ts = 0L;
-	tokctx = NULL;
-
-	// first line of body is either "locked" or "unlocked"
-
-	if (!(line = strtok_r(self->gvfs_input_buffer, "\n", &tokctx))) {
-		g_warning("ccchh-dooris-widget: first line of response not found");
-		fetch_update_process(self, new_state, new_change_ts, new_report_ts);
-		return;
-	}
-
-	if (!strcmp(line, "locked")) {
-		new_state = DOORIS_CLOSED;
-		g_debug("ccchh-dooris-widget: current state: closed");
-	} else if (!strcmp(line, "unlocked")) {
-		new_state = DOORIS_OPEN;
-		g_debug("ccchh-dooris-widget: current state: open");
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value notify_enabled: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
 	} else {
-		g_warning("ccchh-dooris-widget: could not parse response");
-		fetch_update_process(self, new_state, new_change_ts, new_report_ts);
-		return;
+		self->notify = boolval;
+		g_debug("ccchh-dooris-widget: load_settings: notify = %d", self->notify);
 	}
 
-	// second line of body is last sensor report timestamp
 
-	if ((line = strtok_r(NULL, "\n", &tokctx))) {
-		errno = 0;
-		ts = strtoul(line, (char **) NULL, 10);
-		if (ts && !errno) {
-			new_report_ts = ts;
-			g_debug("ccchh-dooris-widget: last_report_ts: %lu, errno: %i\n", ts, errno);
-		} else {
-			g_warning("ccchh-dooris-widget: could not parse last_report_ts");
-		}
+	boolval = gconf_client_get_bool(self->gconf_client,
+		GCONF_DIR"/use_connection", &error);
 
-		// third line of body is last state change timestamp
-
-		if ((line = strtok_r(NULL, "\n", &tokctx))) {
-			errno = 0;
-			ts = strtoul(line, (char **) NULL, 10);
-			if (ts && !errno) {
-				new_change_ts = ts;
-				g_debug("ccchh-dooris-widget: last_change_ts: %lu, errno: %i\n", ts, errno);
-			} else {
-				g_warning("ccchh-dooris-widget: could not parse last_change_ts");
-			}
-		}
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value use_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else {
+		self->use_connection = boolval;
+		g_debug("ccchh-dooris-widget: load_settings: use_connection = %d", self->use_connection);
 	}
 
-/*
-	// TODO: reuse small buffers
-	g_free(self->gvfs_input_buffer);
-	self->gvfs_input_buffer = NULL;
-*/
+	boolval = gconf_client_get_bool(self->gconf_client,
+		GCONF_DIR"/force_connection", &error);
 
-	fetch_update_process(self, new_state, new_change_ts, new_report_ts);
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value force_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else {
+		self->force_connection = boolval;
+		g_debug("ccchh-dooris-widget: load_settings: force_connection = %d", self->force_connection);
+	}
 }
 
 
-// handle async close callback
 static void
-fetch_update_async_close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
-	gpointer data)
+save_settings(CCCHHDoorIsDesktopWidget *self)
 {
-//	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+	GError *error;
 
-	g_debug("ccchh-dooris-widget: fetch_update_async_close_cb()");
+	error = NULL;
 
-	if (result != GNOME_VFS_OK) {
-		g_debug("ccchh-dooris-widget: error on close: %s", gnome_vfs_result_to_string (result));
-		// TODO: what now?
+	g_debug("ccchh-dooris-widget: save_settings()");
+
+	if (!gconf_client_set_string(self->gconf_client,
+		GCONF_DIR"/update_url", self->url, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value update_url: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_int(self->gconf_client,
+		GCONF_DIR"/update_interval", self->update, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value update_interval: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/notify_enabled", self->notify, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value notify_enabled: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/use_connection", self->use_connection, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value use_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/force_connection", self->force_connection, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value force_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
 	}
 }
 
-
-// update internal state, unlock mutex
-static void
-fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
-	gulong new_change_ts, gulong new_report_ts)
-{
-	g_debug("ccchh-dooris-widget: fetch_update_process()");
-
-	self->current_state = new_state;
-	self->last_change_ts = new_change_ts;
-	self->last_report_ts = new_report_ts;
-
-	if (new_state == self->last_state) {
-		g_debug("ccchh-dooris-widget: state unchanged");
-
-		if (self->force_banner) {
-			self->force_banner = FALSE;
-			show_status_banner(self);
-		}
-
-		fetch_update_unlock(self);
-		return;
-	}
-
-	// release old icon surface
-	if (self->image_current) {
-		cairo_surface_destroy(self->image_current);
-		self->image_current = NULL;
-	}
-
-	// set new icon surface
-	switch (self->current_state) {
-		case DOORIS_OPEN:
-			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_OPEN);
-			g_debug("ccchh-dooris-widget: image_current set to open");
-			break;
-		case DOORIS_CLOSED:
-			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_CLOSED);
-			g_debug("ccchh-dooris-widget: image_current set to closed");
-			break;
-		default:
-			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
-			g_debug("ccchh-dooris-widget: image_current set to fnord");
-			break;
-	}
-
-	self->last_state = self->current_state;
-
-	// state is valid and has changed since last known state
-	if (self->current_state != DOORIS_FNORD && self->current_state != self->last_known_state) {
-
-		self->last_known_state = self->current_state;
-		g_debug("ccchh-dooris-widget: state changed");
-
-		// state changed: redraw icon
-		force_redraw(self);
-
-		// flash banner if notifications are enabled
-		if (self->notify) {
-			self->force_banner = TRUE;
-		}
-	}
-
-	if (self->force_banner) {
-		self->force_banner = FALSE;
-		show_status_banner(self);
-	}
-
-	fetch_update_unlock(self);
-}
-
-
-
-
 // stop update timer
 // returns TRUE if timer was stopped
 // returns FALSE if no timer was running
@@ -489,9 +347,15 @@ draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 	}
 
 	cairo_save(cr);
+	
+	// transparent background
+	cairo_set_source_rgba(cr, IMAGE_COLOR_BG, IMAGE_ALPHA_BG);
+	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
 
 	// paint icon
 	cairo_set_source_surface(cr, self->image_current, 5, 5);
+	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
 	cairo_paint_with_alpha(cr, self->current_alpha);
 
 	// if not connected, overlay icon with "disconnected" image
@@ -503,11 +367,10 @@ draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 		cairo_surface_destroy(image_overlay);
 	}
 
-	//
+	// if button pressed, hightligth with white mask
 	if (self->highlighted) {
-		cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 0.6);
+		cairo_set_source_rgba(cr, IMAGE_COLOR_HIGHLIGHT, IMAGE_ALPHA_HIGHLIGHT);
 		cairo_mask_surface(cr, self->image_current, 5, 5);
-//		cairo_paint_with_alpha(cr, self->current_alpha);
 	}
 
 	cairo_restore(cr);
@@ -598,6 +461,268 @@ force_redraw (CCCHHDoorIsDesktopWidget *self)
 // end private functions
 
 
+
+
+// async update with callbacks
+
+// lock fetch mutex, redraw transparent button
+static inline gboolean
+fetch_update_lock (CCCHHDoorIsDesktopWidget *self)
+{
+	if (!g_static_mutex_trylock(&(self->fetch_lock))) {
+		return FALSE;
+	}
+
+	// acknowledge button, make icon transparent
+	self->current_alpha = IMAGE_ALPHA_PROGRESS;
+	force_redraw(self);
+
+	return TRUE;
+}
+
+
+// unlock fetch mutex, redraw opaque button
+static inline void
+fetch_update_unlock (CCCHHDoorIsDesktopWidget *self)
+{
+	self->current_alpha = IMAGE_ALPHA_NORMAL;
+	force_redraw(self);
+
+	g_static_mutex_unlock(&(self->fetch_lock));
+}
+
+
+// initialize async update from server, lock mutex
+static void
+fetch_update_start (CCCHHDoorIsDesktopWidget *self)
+{
+	g_debug("ccchh-dooris-widget: fetch_update_start()");
+
+	if (!self->connected) {
+		g_debug("ccchh-dooris-widget: not connected, not fetching new state");
+		return;
+	}
+
+	if (!fetch_update_lock(self)) {
+		g_message("ccchh-dooris-widget: fetch_lock set, not updating");
+		return;
+	}
+
+	g_debug("ccchh-dooris-widget: begin update from URL: %s", self->url);
+
+	// TODO: set timeout to cancel async open
+
+	gnome_vfs_async_open(&self->gvfs_handle, self->url, GNOME_VFS_OPEN_READ,
+		GNOME_VFS_PRIORITY_MIN, fetch_update_async_open_cb, self);
+}
+
+
+// handle async open callback, begin read
+static void
+fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
+    gpointer data)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+
+	g_debug("ccchh-dooris-widget: fetch_update_async_open_cb()");
+
+	if (result != GNOME_VFS_OK) {
+		g_warning("ccchh-dooris-widget: error opening URL: %s", gnome_vfs_result_to_string (result));
+		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
+		return;
+	}
+
+	// reuse input buffer, ensure it has at least MAX_HTTP_SIZE+1
+	if (!(self->gvfs_input_buffer = g_try_realloc(self->gvfs_input_buffer, MAX_HTTP_SIZE+1))) {
+		g_warning("ccchh-dooris-widget: failed to allocate input buffer");
+		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
+		return;
+	}
+	// initialize input buffer with zeroes, so strtok won't read beyond its end
+	memset(self->gvfs_input_buffer, 0, MAX_HTTP_SIZE+1);
+
+	// TODO: set timeout to cancel async read
+
+	gnome_vfs_async_read (handle, self->gvfs_input_buffer, MAX_HTTP_SIZE,
+		fetch_update_async_read_cb, self);
+}
+
+
+// handle async read callback, begin close, parse input
+static void
+fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
+	gpointer buffer, GnomeVFSFileSize bytes_requested, GnomeVFSFileSize bytes_read,
+	gpointer data)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+
+	g_debug("ccchh-dooris-widget: fetch_update_async_read_cb()");
+
+	if (result != GNOME_VFS_OK) {
+		g_warning("ccchh-dooris-widget: read error: %s", gnome_vfs_result_to_string (result));
+		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
+		return;
+	}
+
+	g_debug("ccchh-dooris-widget: read %"GNOME_VFS_SIZE_FORMAT_STR" bytes", bytes_read);
+
+	// TODO: set timeout to cancel close
+
+	// close file
+	gnome_vfs_async_close (handle, fetch_update_async_close_cb, self);
+
+	// meanwhile, parse input buffer
+
+	gulong ts, new_change_ts, new_report_ts;
+	dooris_state_t new_state;
+	char *line, *tokctx;
+
+	new_state = DOORIS_FNORD;
+	new_change_ts = 0L;
+	new_report_ts = 0L;
+	tokctx = NULL;
+
+	// first line of body is either "locked" or "unlocked"
+
+	if (!(line = strtok_r(self->gvfs_input_buffer, "\n", &tokctx))) {
+		g_warning("ccchh-dooris-widget: first line of response not found");
+		fetch_update_process(self, new_state, new_change_ts, new_report_ts);
+		return;
+	}
+
+	if (!strcmp(line, "locked")) {
+		new_state = DOORIS_CLOSED;
+		g_debug("ccchh-dooris-widget: current state: closed");
+	} else if (!strcmp(line, "unlocked")) {
+		new_state = DOORIS_OPEN;
+		g_debug("ccchh-dooris-widget: current state: open");
+	} else {
+		g_warning("ccchh-dooris-widget: could not parse response");
+		fetch_update_process(self, new_state, new_change_ts, new_report_ts);
+		return;
+	}
+
+	// second line of body is last sensor report timestamp
+
+	if ((line = strtok_r(NULL, "\n", &tokctx))) {
+		errno = 0;
+		ts = strtoul(line, (char **) NULL, 10);
+		if (ts && !errno) {
+			new_report_ts = ts;
+			g_debug("ccchh-dooris-widget: last_report_ts: %lu, errno: %i\n", ts, errno);
+		} else {
+			g_warning("ccchh-dooris-widget: could not parse last_report_ts");
+		}
+
+		// third line of body is last state change timestamp
+
+		if ((line = strtok_r(NULL, "\n", &tokctx))) {
+			errno = 0;
+			ts = strtoul(line, (char **) NULL, 10);
+			if (ts && !errno) {
+				new_change_ts = ts;
+				g_debug("ccchh-dooris-widget: last_change_ts: %lu, errno: %i\n", ts, errno);
+			} else {
+				g_warning("ccchh-dooris-widget: could not parse last_change_ts");
+			}
+		}
+	}
+
+	fetch_update_process(self, new_state, new_change_ts, new_report_ts);
+}
+
+
+// handle async close callback
+static void
+fetch_update_async_close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
+	gpointer data)
+{
+//	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+
+	g_debug("ccchh-dooris-widget: fetch_update_async_close_cb()");
+
+	if (result != GNOME_VFS_OK) {
+		g_debug("ccchh-dooris-widget: error on close: %s", gnome_vfs_result_to_string (result));
+		// TODO: what now?
+	}
+}
+
+
+// update internal state, unlock mutex
+static void
+fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
+	gulong new_change_ts, gulong new_report_ts)
+{
+	g_debug("ccchh-dooris-widget: fetch_update_process()");
+
+	self->current_state = new_state;
+	self->last_change_ts = new_change_ts;
+	self->last_report_ts = new_report_ts;
+
+	if (new_state == self->last_state) {
+		g_debug("ccchh-dooris-widget: state unchanged");
+
+		if (self->force_banner) {
+			self->force_banner = FALSE;
+			show_status_banner(self);
+		}
+
+		fetch_update_unlock(self);
+		return;
+	}
+
+	// release old icon surface
+	if (self->image_current) {
+		cairo_surface_destroy(self->image_current);
+		self->image_current = NULL;
+	}
+
+	// set new icon surface
+	switch (self->current_state) {
+		case DOORIS_OPEN:
+			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_OPEN);
+			g_debug("ccchh-dooris-widget: image_current set to open");
+			break;
+		case DOORIS_CLOSED:
+			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_CLOSED);
+			g_debug("ccchh-dooris-widget: image_current set to closed");
+			break;
+		default:
+			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
+			g_debug("ccchh-dooris-widget: image_current set to fnord");
+			break;
+	}
+
+	self->last_state = self->current_state;
+
+	// state is valid and has changed since last known state
+	if (self->current_state != DOORIS_FNORD && self->current_state != self->last_known_state) {
+
+		self->last_known_state = self->current_state;
+		g_debug("ccchh-dooris-widget: state changed");
+
+		// state changed: redraw icon
+		force_redraw(self);
+
+		// flash banner if notifications are enabled
+		if (self->notify) {
+			self->force_banner = TRUE;
+		}
+	}
+
+	if (self->force_banner) {
+		self->force_banner = FALSE;
+		show_status_banner(self);
+	}
+
+	fetch_update_unlock(self);
+}
+
+
+
+
+// event handlers
+
 static gboolean
 ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 {
@@ -644,6 +769,7 @@ ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *eve
 	draw (CCCHH_DOORIS_DESKTOP_WIDGET (widget), cr);
 
 	cairo_destroy (cr);
+
 	return FALSE;
 }
 
@@ -669,6 +795,7 @@ ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget,
 	return TRUE; /* Prevent further handling of this signal. */
 }
 
+
 // handle the leave notify event, remove highlight
 static gboolean 
 ccchh_dooris_desktop_widget_on_button_leave_event (GtkWidget *widget, 
@@ -731,19 +858,19 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
   ConIcConnectionEvent *event, gpointer user_data)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (user_data);
-	const gchar *iap_id = con_ic_event_get_iap_id(CON_IC_EVENT(event));
 
 	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_connection_event()");
 
 	// libconic signalled a connection state change
 
 	ConIcConnectionStatus status = con_ic_connection_event_get_status(event);
+	const gchar *iap_id = con_ic_event_get_iap_id(CON_IC_EVENT(event));
 
 	// Problem: When switching from GPRS to/from WLAN
 	// we receive STATUS_CONNECTED for the new IAP first,
 	// then STATUS_DISCONNECTED for the previous IAP
 
-	// in STATUS_DISCONNECTING, save disconneting IAP ID and
+	// in STATUS_DISCONNECTING, save disconnecting IAP ID and
 	// set self->connected to FALSE
 	// STATUS_CONNECTED sets self->connected to TRUE
 	// if STATUS_DISCONNECTED is called with the same IAP ID
@@ -821,8 +948,6 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 }
 
 
-// TODO: save settings
-
 static void 
 ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
   gint response_id, gpointer user_data)
@@ -892,10 +1017,14 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 		} else {
 			g_debug("ccchh-dooris-widget: button does not force connection");
 		}
+
+		save_settings(self);
 	}
 
 	gtk_widget_destroy (self->settings_window);
-	// TODO: isn't there a better way to this? array+memset?
+	// TODO: does this leak any object references such as GtkSizeGroup
+	// or HildonCaption?
+	// isn't there a better way to this? array+memset?
 	self->settings_window = NULL;
 	self->url_entry = NULL;
 	self->update_picker = NULL;
@@ -950,13 +1079,13 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		GtkWidget *hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-		// TODO: use HildonCaption
+		GtkSizeGroup *sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 
 		// status url
-		GtkWidget *label = gtk_label_new (_("Status URL:"));
-		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 		self->url_entry = hildon_entry_new (HILDON_SIZE_FINGER_HEIGHT);
-		gtk_box_pack_start (GTK_BOX (hbox), self->url_entry, TRUE, TRUE, 0);
+		GtkWidget *caption = hildon_caption_new(NULL, _("Status URL:"),
+			self->url_entry, NULL, HILDON_CAPTION_MANDATORY);
+		gtk_box_pack_start (GTK_BOX (hbox), caption, TRUE, TRUE, 0);
 
 		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -979,18 +1108,17 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		// install callback function to validate input
 		g_signal_connect (G_OBJECT (self->update_picker), "value-changed",
 			G_CALLBACK (update_picker_changed_value_cb), NULL);
-		// TODO: this doesn't look good, really use HildonCaption
-		hildon_button_set_title(HILDON_BUTTON (self->update_picker), _("Update interval (minutes):"));
 		// align left, center vertically
 		gtk_button_set_alignment(GTK_BUTTON(self->update_picker), 0, 0.5);
-		gtk_box_pack_start (GTK_BOX (hbox), self->update_picker, TRUE, TRUE, 0);
+		caption = hildon_caption_new(sizegroup, _("Update interval (minutes):"),
+			self->update_picker, NULL, HILDON_CAPTION_MANDATORY);
+		gtk_box_pack_start (GTK_BOX (hbox), caption, TRUE, TRUE, 0);
 
 		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
     
 		// banner notifications
 		self->notify_button = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
-		// TODO: this doesn't look good, really use HildonCaption
 		gtk_button_set_label(GTK_BUTTON (self->notify_button), _("Notify on state change"));
 		// align left, center vertically
 		gtk_button_set_alignment(GTK_BUTTON(self->notify_button), 0, 0.5);
@@ -1013,6 +1141,8 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_button_set_label(GTK_BUTTON (self->connforce_button), _("Button requests internet connection"));
 		gtk_button_set_alignment(GTK_BUTTON(self->connforce_button), 0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox), self->connforce_button, TRUE, TRUE, 0);
+
+		g_object_unref(sizegroup);
 	}
 
 	// set current values
@@ -1033,6 +1163,8 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 }
 
 
+
+
 static void
 ccchh_dooris_desktop_widget_realize (GtkWidget *widget)
 {
@@ -1111,6 +1243,15 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 		self->gvfs_input_buffer = NULL;
 	}
 
+	if (self->disconnecting) {
+		g_free(self->disconnecting);
+		self->disconnecting = NULL;
+	}
+
+	if (self->gconf_client) {
+		g_object_unref(self->gconf_client);
+		self->gconf_client = NULL;
+	}
 
 	/* Call the base class's implementation: */
 	G_OBJECT_CLASS (ccchh_dooris_desktop_widget_parent_class)->finalize (object);
@@ -1184,6 +1325,8 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	g_static_mutex_init(&(self->button_lock));
 	g_static_mutex_init(&(self->fetch_lock));
 
+	self->gconf_client = gconf_client_get_default();
+
 	self->force_banner = FALSE;
 
 	self->disconnecting = NULL;
@@ -1220,6 +1363,10 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	g_signal_connect (self, "show-settings", 
 		G_CALLBACK (ccchh_dooris_desktop_widget_on_show_settings), NULL);   
 
+	if (gconf_client_dir_exists(self->gconf_client, GCONF_DIR, NULL)) {
+		load_settings(self);
+	}
+
 	/* Create connection object */
 	self->connection = con_ic_connection_new ();
 	self->connection_handler = 0;
