@@ -4,33 +4,43 @@
 #define _CCCHH_DOORIS_DESKTOP_WIDGET
 
 
+// gconf base path
+#define GCONF_DIR "/apps/maemo/ccchh-dooris-widget"
+
 // max download size
 #define MAX_HTTP_SIZE 1024
 
 // minimum/maximum selectable update interval
 // in minutes
-#define UPDATE_INTERVAL_MIN 5
-#define UPDATE_INTERVAL_MAX 1440
+#define UPDATE_INTERVAL_MIN	5
+#define UPDATE_INTERVAL_MAX	1440
 
 // paths to icon files
-#define IMAGE_DOORIS_OPEN           DATADIR"/dooris_open.png"
-#define IMAGE_DOORIS_CLOSED         DATADIR"/dooris_closed.png"
-#define IMAGE_DOORIS_FNORD          DATADIR"/dooris_fnord.png"
-#define IMAGE_OVERLAY_DISCONNECTED  DATADIR"/dooris_notavail.png"
+#define IMAGE_DOORIS_OPEN			DATADIR"/dooris_open.png"
+#define IMAGE_DOORIS_CLOSED			DATADIR"/dooris_closed.png"
+#define IMAGE_DOORIS_FNORD			DATADIR"/dooris_fnord.png"
+#define IMAGE_OVERLAY_DISCONNECTED	DATADIR"/dooris_notavail.png"
 
 // widget size
-#define IMAGE_WIDGET_WIDTH  102
-#define IMAGE_WIDGET_HEIGHT 72
+#define IMAGE_WIDGET_WIDTH	102
+#define IMAGE_WIDGET_HEIGHT	72
 
-// transparency
-#define IMAGE_ALPHA_NORMAL      1.0
-#define IMAGE_ALPHA_HIGHLIGHT   0.65
+// colors & transparency
+#define IMAGE_COLOR_BG			1, 1, 1
+#define IMAGE_ALPHA_BG			0.0
+
+#define IMAGE_COLOR_HIGHLIGHT	1, 1, 1
+#define IMAGE_ALPHA_HIGHLIGHT	0.65
+
+#define IMAGE_ALPHA_NORMAL		1.0
+#define IMAGE_ALPHA_PROGRESS	0.65
 
 
 G_BEGIN_DECLS
 
 #include <libhildondesktop/libhildondesktop.h>
 #include <conic.h> /* libconic - Internet Connectivity library */
+#include <gconf/gconf-client.h>
 
 #define TYPE_CCCHH_DOORIS_DESKTOP_WIDGET ccchh_dooris_desktop_widget_get_type()
 
@@ -71,6 +81,7 @@ typedef struct {
 	gdouble current_alpha;
 	GStaticMutex button_lock;
 	GStaticMutex fetch_lock;
+	GConfClient *gconf_client;
 
 	// door state information
 	dooris_state_t last_state;
